@@ -1,11 +1,11 @@
- #!/usr/bin/python
+#!/usr/bin/python
 from __future__ import division
 from copy import copy
 import pickle
 import sys
 
 import numpy as np
-from scipy.optimize import fmin_powell, fmin
+from scipy.optimize import minimize, basinhopping
 
 from collections import OrderedDict, defaultdict
 
@@ -14,6 +14,7 @@ import pymc as pm
 import warnings
 
 from kabuki.utils import flatten
+import analyze
 
 class LnProb(object):
     def __init__(self, model):
@@ -387,6 +388,10 @@ class Hierarchical(object):
         self._setup_model()
         self.create_model()
 
+        # backwards compat
+        if not hasattr(self, 'sampled'):
+            self.sampled = True
+
         if self.sampled:
             self.load_db(d['dbname'], db=d['db'])
             self.gen_stats()
@@ -442,16 +447,25 @@ class Hierarchical(object):
     def create_nodes_db(self):
         self.nodes_db = pd.concat([knode.nodes_db for knode in self.knodes])
 
-    def draw_from_prior(self):
-        non_zero = False
+    def draw_from_prior(self, update=False):
+        if not update:
+            values = self.values
+
+        non_zero = True
         while non_zero:
             try:
                 self.mc.draw_from_prior()
                 self.mc.logp
-                non_zero = True
-            except pm.ZeroProbability:
+                draw = copy(self.values)
                 non_zero = False
+            except pm.ZeroProbability:
+                non_zero = True
 
+        if not update:
+            # restore original values
+            self.set_values(values)
+
+        return draw
 
     def map(self, runs=2, warn_crit=5, method='fmin_powell', **kwargs):
         """
@@ -486,13 +500,15 @@ class Hierarchical(object):
         for i in range(runs):
             # (re)create nodes to get new initival values.
             #nodes are not created for the first iteration if they already exist
+            self.mc = pm.MAP(self.nodes_db.node)
             if i != 0:
                 self.draw_from_prior()
 
-            m = pm.MAP(self.nodes_db.node)
-            m.fit(method, **kwargs)
-            print m.logp
-            maps.append(m)
+            self.mc.fit(method, **kwargs)
+            print self.mc.logp
+            maps.append(self.mc)
+
+        self.mc = None
 
         # We want to use values of the best fitting model
         sorted_maps = sorted(maps, key=attrgetter('logp'))
@@ -636,8 +652,15 @@ class Hierarchical(object):
 
         self.sampled = True
 
+        self.gen_stats()
         return self.mc
 
+    @property
+    def logp(self):
+        if self.mc is None:
+            raise AttributeError('self.mc not set. Call mcmc().')
+        return self.mc.logp
+
     @property
     def dic_info(self):
         """returns information about the model DIC."""
@@ -727,29 +750,22 @@ class Hierarchical(object):
         else:
             i_chain = nchains
 
-        #see if stats have been cached for this chain
-        try:
-            if self._stats_chain == i_chain:
-                return
-        except AttributeError:
-            pass
-
         #update self._stats
         self._stats = self.mc.stats(*args, **kwargs)
         self._stats_chain = i_chain
 
         #add/overwrite stats to nodes_db
         for name, i_stats in self._stats.iteritems():
-            if self.nodes_db['hidden'][name]:
+            if self.nodes_db.loc[name, 'hidden']:
                 continue
-            self.nodes_db['mean'][name]   = i_stats['mean']
-            self.nodes_db['std'][name]    = i_stats['standard deviation']
-            self.nodes_db['2.5q'][name]   = i_stats['quantiles'][2.5]
-            self.nodes_db['25q'][name]    = i_stats['quantiles'][25]
-            self.nodes_db['50q'][name]    = i_stats['quantiles'][50]
-            self.nodes_db['75q'][name]    = i_stats['quantiles'][75]
-            self.nodes_db['97.5q'][name]  = i_stats['quantiles'][97.5]
-            self.nodes_db['mc err'][name] = i_stats['mc error']
+            self.nodes_db.loc[name, 'mean']   = i_stats['mean']
+            self.nodes_db.loc[name, 'std']    = i_stats['standard deviation']
+            self.nodes_db.loc[name, '2.5q']   = i_stats['quantiles'][2.5]
+            self.nodes_db.loc[name, '25q']    = i_stats['quantiles'][25]
+            self.nodes_db.loc[name, '50q']    = i_stats['quantiles'][50]
+            self.nodes_db.loc[name, '75q']    = i_stats['quantiles'][75]
+            self.nodes_db.loc[name, '97.5q']  = i_stats['quantiles'][97.5]
+            self.nodes_db.loc[name, 'mc err'] = i_stats['mc error']
 
 
     def load_db(self, dbname, verbose=0, db='sqlite'):
@@ -802,24 +818,37 @@ class Hierarchical(object):
         which will change the plot attribute in the relevant nodes
         """
 
-        #should we save the figures
+        # should we save the figures
         kwargs.pop('last', None)
 
         if isinstance(params, str):
              params = [params]
 
-        #loop over nodes and for each node if it
+        # loop over nodes and for each node if it
         for (name, node) in self.iter_non_observeds():
-            if (params is None) or (node['knode_name'] in params): #plot params if its name was mentioned
-                if not node['hidden']: #plot it if it is not hidden
+            if (params is None) or (node['knode_name'] in params): # plot params if its name was mentioned
+                if not node['hidden']: # plot it if it is not hidden
                     plot_value = node['node'].plot
-                    if (plot_subjs and node['subj']): #plot if it is a subj node and plot_subjs==True
+                    if (plot_subjs and node['subj']): # plot if it is a subj node and plot_subjs==True
                         node['node'].plot = True
-                    if (params is not None) and  (node['knode_name'] in params): #plot if it was sepecficily mentioned
+                    if (params is not None) and  (node['knode_name'] in params): # plot if it was sepecficily mentioned
                         node['node'].plot = True
                     pm.Matplot.plot(node['node'], last=save, **kwargs)
                     node['node'].plot = plot_value
 
+    def plot_posteriors_conditions(self, *args, **kwargs):
+        """
+        Plot all group posteriors listed in depends_on on individual graphs.
+
+        Forwards arguments to kabuki.analyze.plot_posterior_nodes.
+        """
+        group_nodes = self.get_group_nodes()
+        for dep in self.depends_on.iterkeys():
+            nodes = group_nodes.ix[group_nodes.knode_name == dep]
+            if all(nodes.hidden == True):
+                continue
+            analyze.plot_posterior_nodes(nodes['node'], *args, **kwargs)
+
     def get_observeds(self):
         return self.nodes_db[self.nodes_db.observed == True]
 
@@ -898,7 +927,12 @@ class Hierarchical(object):
 
     @property
     def values(self):
-        return {name: node['node'].value[()] for (name, node) in self.iter_non_observeds() if node['node'].value.shape == ()}
+        values = OrderedDict()
+        for (name, node) in self.iter_non_observeds():
+            if node['node'].value.shape == ():
+                values[name] = node['node'].value[()]
+
+        return values
 
     def set_values(self, new_values):
         """
@@ -909,53 +943,105 @@ class Hierarchical(object):
         for (name, value) in new_values.iteritems():
             self.nodes_db.ix[name]['node'].set_value(value)
 
-    def find_starting_values(self):
+    def find_starting_values(self, *args, **kwargs):
         """Find good starting values for the different parameters by
         optimization.
+
+        For more options see approximate_map and map. Arguments are forwarded.
         """
         if self.is_group_model:
-            self.approximate_map()
+            self.approximate_map(*args, **kwargs)
         else:
-            self.map()
+            self.map(*args, **kwargs)
 
-    def _partial_optimize(self, optimize_nodes, evaluate_nodes, fall_to_simplex):
+    def _partial_optimize(self, optimize_nodes, evaluate_nodes, fall_to_simplex=True, minimizer='Powell', use_basin=False, debug=False, minimizer_kwargs=None, basin_kwargs=None):
         """Optimize part of the model.
 
         :Arguments:
             nodes : iterable
                 list nodes to optimize.
         """
+        if minimizer_kwargs is None:
+            minimizer_kwargs = {}
+        if basin_kwargs is None:
+            basin_kwargs = {}
+
         non_observeds = filter(lambda x: not x.observed, optimize_nodes)
 
         init_vals = [node.value for node in non_observeds]
 
         # define function to be optimized
         def opt(values):
+            if debug: print(values)
             for value, node in zip(values, optimize_nodes):
                 node.set_value(value)
             try:
                 logp_optimize = [node.logp for node in optimize_nodes]
                 logp_evaluate = [node.logp for node in evaluate_nodes]
-                return -np.sum(logp_optimize) - np.sum(logp_evaluate)
+                neglogp = -np.sum(logp_optimize) - np.sum(logp_evaluate)
+                if debug: print(neglogp)
+                return neglogp
             except pm.ZeroProbability:
+                if debug: print('Outside support!')
                 return np.inf
 
-        #optimize
-        try:
-            fmin_powell(opt, init_vals)
-        except Exception as e:
-            if fall_to_simplex:
-                print "Warning: Powell optimization failed. Falling back to simplex."
-                fmin(opt, init_vals)
-            else:
-                raise e
+        # optimize
+        if use_basin:
+            try:
+                minimizer_kwargs_passed = {'method': minimizer, 'options': minimizer_kwargs}
+                basinhopping(opt, init_vals, minimizer_kwargs=minimizer_kwargs_passed, **basin_kwargs)
+            except:
+                if fall_to_simplex:
+                    print("Warning: Powell optimization failed. Falling back to simplex.")
+                    minimizer_kwargs_passed = {'method': minimizer, 'options': minimizer_kwargs}
+                    basinhopping(opt, init_vals, minimizer_kwargs=minimizer_kwargs_passed, **basin_kwargs)
+                else:
+                    raise
+        else:
+            try:
+                minimize(opt, init_vals, method=minimizer, options=minimizer_kwargs)
+            except:
+                if fall_to_simplex:
+                    print("Warning: Powell optimization failed. Falling back to simplex.")
+                    minimize(opt, init_vals, method='Nelder-Mead', options=minimizer_kwargs)
+                else:
+                    raise
 
 
-    def approximate_map(self, fall_to_simplex = True):
+    def _approximate_map_subj(self, minimizer='Powell', use_basin=False, fall_to_simplex=True, debug=False, minimizer_kwargs=None, basin_kwargs=None):
+        # Optimize subj nodes
+        for subj_idx in self.nodes_db.subj_idx.dropna().unique():
+            stoch_nodes = self.nodes_db.ix[(self.nodes_db.subj_idx == subj_idx) & (self.nodes_db.stochastic == True)].node
+            obs_nodes = self.nodes_db.ix[(self.nodes_db.subj_idx == subj_idx) & (self.nodes_db.observed == True)].node
+            self._partial_optimize(stoch_nodes, obs_nodes, fall_to_simplex=fall_to_simplex, minimizer=minimizer, use_basin=use_basin, debug=debug, minimizer_kwargs=minimizer_kwargs, basin_kwargs=basin_kwargs)
+
+    def approximate_map(self, individual_subjs=True, minimizer='Powell', use_basin=False, fall_to_simplex=True, cycles=1, debug=False, minimizer_kwargs=None, basin_kwargs=None):
         """Set model to its approximate MAP.
-        Input:
-            fall_to_simplex <bool>
+
+        :Arguments:
+            individual_subjs : bool <default=True>
+                Optimize each subject individually.
+            minimizer : str <default='Powell'>
+                Optimize using Powell. See numpy.optimize.minimize.
+                Other choice might be 'Nelder-Mead'
+            use_basin : bool <default=True>
+                Use basin hopping optimization to avoid local minima.
+            fall_to_simplex : bool <default=True>
                 should map try using simplex algorithm if powell method failes
+            cycles : int <default=1>
+                How many times to optimize the model.
+                Since lower level nodes depend on higher level nodes,
+                they might be estimated differently in a second pass.
+            minimizer_kwargs : dict <default={}>
+                Keyword arguments passed to minimizer.
+                See scipy.optimize.minimize for options.
+            basin_kwargs : dict <default={}>
+                Keyword arguments passed to basinhopping.
+                See scipy.optimize.basinhopping for options.
+            debug : bool <default=False>
+                Whether to print current values and neg logp at each
+                iteration.
+
         """
         ###############################
         # In order to find the MAP of a hierarchical model one needs
@@ -965,12 +1051,23 @@ class Hierarchical(object):
         # only need this to get at the generations
         # TODO: Find out how to get this from pymc.utils.find_generations()
         m = pm.MCMC(self.nodes_db.node)
-        generations = m.generations
-        generations.append(self.get_observeds().node)
+        generations_unsorted = m.generations
+        generations_unsorted.append(self.get_observeds().node)
+        # Filter out empty generations
+        generations_unsorted = [gen for gen in generations_unsorted if len(gen) != 0]
+        # Sort generations according to order of nodes_db
+        generations = []
+        for gen in generations_unsorted:
+            generations.append([row.node for name, row in self.nodes_db.iterrows()
+                                if name in [node.__name__ for node in gen]])
 
-        for i in range(len(generations)-1, 0, -1):
-            # Optimize the generation at i-1 evaluated over the generation at i
-            self._partial_optimize(generations[i-1], generations[i], fall_to_simplex)
+        for cyc in range(cycles):
+            for i in range(len(generations)-1, 0, -1):
+                if self.is_group_model and individual_subjs and (i == len(generations) - 1):
+                    self._approximate_map_subj(fall_to_simplex=fall_to_simplex, minimizer=minimizer, use_basin=use_basin, debug=debug, minimizer_kwargs=minimizer_kwargs, basin_kwargs=basin_kwargs)
+                    continue
+                # Optimize the generation at i-1 evaluated over the generation at i
+                self._partial_optimize(generations[i-1], generations[i], fall_to_simplex=fall_to_simplex, minimizer=minimizer, use_basin=use_basin, debug=debug, minimizer_kwargs=minimizer_kwargs, basin_kwargs=basin_kwargs)
 
         #update map in nodes_db
         self.nodes_db['map'] = np.NaN
